window.Gleam.initSearch([{"doc":"valid","title":"valid","content":"# Valid\n\n![CI](https://github.com/sporto/gleam-valid/workflows/test/badge.svg?branch=main)\n\nA validation library for [Gleam](https://gleam.run/).\n\nAPI Docs: &lt;https://hexdocs.pm/valid&gt;.\n\nThis library follows the principle [Parse don&#39;t validate](https://lexi-lambda.github.io/blog/2019/11/05/parse-don-t-validate/).\n\n## Install\n\n```\ngleam add valid\n```\n\n## Usage\n\nYou start with an input type and validate into an output type. These two types can be different. For example:\n\n```gleam\ntype UserInput { UserInput(name: Option(String), age: Int) }\n\ntype ValidUser { ValidUser(name: String, age: Int) }\n```\n\nThen you create a validator like:\n\n```gleam\nimport valid.{type ValidatorResult}\n\nfn user_validator(user: UserInput) -&gt; ValidatorResult(ValidUser, String) {\n  valid.build2(ValidUser)\n  |&gt; valid.check(user.name, valid.is_some(&quot;Please provide a name&quot;))\n  |&gt; valid.check(user.age, valid.int_min(13, &quot;Must be at least 13 years old&quot;))\n}\n```\n\nAnd run it:\n\n```gleam\ncase user_valid(input) {\n  Ok(valid_user) -&gt; ...\n  Error(tuple(first_error, all_errors)) -&gt; ...\n}\n```\n\n## Error type\n\nErrors can be your own type e.g.\n\n```gleam\nimport valid.{type ValidatorResult}\n\ntype Error {\n  ErrorEmptyName,\n  ErrorTooYoung,\n}\n\nfn user_valid(user: UserInput) -&gt; ValidatorResult(ValidUser, String) {\n  valid.build2(ValidUser)\n  |&gt; valid.check(user.name, valid.is_some(ErrorEmptyName))\n  |&gt; valid.check(user.age, valid.int_min(13, ErrorTooYoung))\n}\n```\n\n## ValidatorResult\n\n`ValidatorResult(valid, error)` is an alias for `Result(valid, tuple(error, List(error)))`\n\nThe `Ok` branch has the valid output.\n\nThe `Error` branch has a tuple `tuple(error, List(error))`.\n\nThe first value is the first error. The second value is a list with all errors (including the first).\n\n## Validators\n\nSee the [API Docs](https://hexdocs.pm/valid/) for the list of included validators.\n\n## Custom property validator\n\nA property validator has two components:\n\n- The error to return\n- A function that transforms the property if successful (`fn(input) -&gt; Option(output)`)\n\nExample:\n\n```gleam\nimport gleam/option.{None, Option, Some}\nimport valid\n\nfn bigger_than_10(num: Int) -&gt; Option(num) {\n  case num &gt; 10 {\n    True -&gt;\n      Some(num)\n    False -&gt;\n      None\n  }\n}\n\nlet custom = valid.custom(&quot;Must be bigger than 10&quot;, bigger_than_10)\n\nlet valid = fn(form: FormInput) {\n  valid.build1(ValidForm)\n  |&gt; valid.check(form.quantity, custom)\n}\n```\n\n## Validating a Dictionary\n\nUse `required_in_dict` and `optional_in_dict`\n\n```gleam\nfn user_validator(dictionary: Dict(String, String)) {\n  valid.build2(ValidUser)\n  |&gt; valid.check(\n    dictionary,\n    valid.required_in_dict(&quot;name&quot;, &quot;Missing name&quot;)\n      |&gt; valid.and_string_is_not_empty(&quot;Please provide a name&quot;),\n  )\n  |&gt; valid.check(\n    input,\n    valid.optional_in_dict(&quot;age&quot;)\n      |&gt; valid.and_optional(valid.string_is_int(&quot;Please provide a valid number&quot;)),\n  )\n}\n```\n\n## Examples\n\nSee [the tests](https://github.com/sporto/gleam-valid/blob/main/test/valid_test.gleam) for many examples\n","url":"index.html"},{"doc":"valid","title":"Errors","content":"pub type Errors(error) =\n  #(error, List(error))\n Error type returned by the validator.\n\n This is a # with the first error and a list of all errors.\n The list includes the first error.\n\nSynonyms:\nvalid.Errors\nvalid Errors","url":"valid.html#Errors"},{"doc":"valid","title":"Validator","content":"pub type Validator(input, output, error) =\n  fn(input) -&gt; ValidatorResult(output, error)\n A Validator is a function that takes an input and\n returns a ValidatorResult\n\nSynonyms:\nvalid.Validator\nvalid Validator","url":"valid.html#Validator"},{"doc":"valid","title":"ValidatorResult","content":"pub type ValidatorResult(output, error) =\n  Result(output, Errors(error))\n\n\nSynonyms:\nvalid.ValidatorResult\nvalid ValidatorResult","url":"valid.html#ValidatorResult"},{"doc":"valid","title":"all","content":"pub fn all(\n  validators: List(fn(a) -&gt; Result(a, #(b, List(b)))),\n) -&gt; fn(a) -&gt; Result(a, #(b, List(b)))\n Validate a value using a list of validators.\n This runs all the validators in the list.\n\n The initial input is passed to all validators.\n All these validators must have the same input and output types.\n\n Returns Ok when all validators pass.\n Returns Error when any validator fails. Error will have all failures.\n\n ## Example\n\n\tlet name_validator = valid.all([\n\t\tvalid.string_is_not_empty(&quot;Empty&quot;),\n\t\tvalid.string_min_length(&quot;&gt;=3&quot;, 3),\n\t\tvalid.string_max_length(&quot;&lt;=10&quot;, 10)\n\t])\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build1(person)\n\t\t|&gt; valid.check(person.name, name_validator)\n\t}\nSynonyms:\nvalid.all\nvalid all","url":"valid.html#all"},{"doc":"valid","title":"and","content":"pub fn and(\n  validator1: fn(a) -&gt; Result(b, #(c, List(c))),\n  validator2: fn(b) -&gt; Result(d, #(c, List(c))),\n) -&gt; fn(a) -&gt; Result(d, #(c, List(c)))\n Compose validators\n\n Run the first validator and if successful then the second.\n Only returns the first error.\n\n ## Example\n\n\tlet name_validator = valid.string_is_not_empty(&quot;Empty&quot;)\n\t|&gt; valid.and(valid.string_min_length(&quot;Must be at least six&quot;, 6))\nSynonyms:\nvalid.and\nvalid and","url":"valid.html#and"},{"doc":"valid","title":"and_int_min","content":"pub fn and_int_min(\n  error: a,\n  min: Int,\n) -&gt; fn(fn(b) -&gt; Result(Int, #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(Int, #(a, List(a)))\n\nSynonyms:\nvalid.and_int_min\nvalid and_int_min","url":"valid.html#and_int_min"},{"doc":"valid","title":"and_list_is_not_empty","content":"pub fn and_list_is_not_empty(\n  error: a,\n) -&gt; fn(fn(b) -&gt; Result(List(c), #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(List(c), #(a, List(a)))\n\nSynonyms:\nvalid.and_list_is_not_empty\nvalid and_list_is_not_empty","url":"valid.html#and_list_is_not_empty"},{"doc":"valid","title":"and_list_max_length","content":"pub fn and_list_max_length(\n  error: a,\n  max: Int,\n) -&gt; fn(fn(b) -&gt; Result(List(c), #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(List(c), #(a, List(a)))\n\nSynonyms:\nvalid.and_list_max_length\nvalid and_list_max_length","url":"valid.html#and_list_max_length"},{"doc":"valid","title":"and_list_min_length","content":"pub fn and_list_min_length(\n  error: a,\n  min: Int,\n) -&gt; fn(fn(b) -&gt; Result(List(c), #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(List(c), #(a, List(a)))\n\nSynonyms:\nvalid.and_list_min_length\nvalid and_list_min_length","url":"valid.html#and_list_min_length"},{"doc":"valid","title":"and_optional","content":"pub fn and_optional(\n  validator: fn(a) -&gt; Result(b, #(c, List(c))),\n) -&gt; fn(fn(d) -&gt; Result(Option(a), #(c, List(c)))) -&gt;\n  fn(d) -&gt; Result(Option(b), #(c, List(c)))\n\nSynonyms:\nvalid.and_optional\nvalid and_optional","url":"valid.html#and_optional"},{"doc":"valid","title":"and_string_is_email","content":"pub fn and_string_is_email(\n  error: a,\n) -&gt; fn(fn(b) -&gt; Result(String, #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(String, #(a, List(a)))\n\nSynonyms:\nvalid.and_string_is_email\nvalid and_string_is_email","url":"valid.html#and_string_is_email"},{"doc":"valid","title":"and_string_is_int","content":"pub fn and_string_is_int(\n  error: a,\n) -&gt; fn(fn(b) -&gt; Result(String, #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(Int, #(a, List(a)))\n\nSynonyms:\nvalid.and_string_is_int\nvalid and_string_is_int","url":"valid.html#and_string_is_int"},{"doc":"valid","title":"and_string_is_not_empty","content":"pub fn and_string_is_not_empty(\n  error: a,\n) -&gt; fn(fn(b) -&gt; Result(String, #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(String, #(a, List(a)))\n\nSynonyms:\nvalid.and_string_is_not_empty\nvalid and_string_is_not_empty","url":"valid.html#and_string_is_not_empty"},{"doc":"valid","title":"and_string_max_length","content":"pub fn and_string_max_length(\n  error: a,\n  max: Int,\n) -&gt; fn(fn(b) -&gt; Result(String, #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(String, #(a, List(a)))\n\nSynonyms:\nvalid.and_string_max_length\nvalid and_string_max_length","url":"valid.html#and_string_max_length"},{"doc":"valid","title":"and_string_min_length","content":"pub fn and_string_min_length(\n  error: a,\n  min: Int,\n) -&gt; fn(fn(b) -&gt; Result(String, #(a, List(a)))) -&gt;\n  fn(b) -&gt; Result(String, #(a, List(a)))\n\nSynonyms:\nvalid.and_string_min_length\nvalid and_string_min_length","url":"valid.html#and_string_min_length"},{"doc":"valid","title":"build1","content":"pub fn build1(constructor: a) -&gt; Result(a, b)\n Build a validator for a type that has one attribute\n\n ## Example\n\n\ttype Person { Person(name: String) }\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build1(person)\n\t\t|&gt; valid.check(person.name, name_validator)\n\t}\nSynonyms:\nvalid.build1\nvalid build1","url":"valid.html#build1"},{"doc":"valid","title":"build2","content":"pub fn build2(\n  constructor: fn(a, b) -&gt; c,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; c, d)\n Build a validator for a type that has two attributes\n\n ## Example\n\n\ttype Person { Person(name: String, age: Int) }\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build2(person)\n\t\t|&gt; valid.check(person.name, name_validator)\n\t\t|&gt; valid.check(person.age, ...)\n\t}\nSynonyms:\nvalid.build2\nvalid build2","url":"valid.html#build2"},{"doc":"valid","title":"build3","content":"pub fn build3(\n  constructor: fn(a, b, c) -&gt; d,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; d, e)\n Build a validator for a type that has three attributes\n\n ## Example\n\n\ttype Person { Person(name: String, age: Int, email: String) }\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build3(person)\n\t\t|&gt; valid.check(person.name, name_validator)\n\t\t|&gt; valid.check(person.age, ...)\n\t\t|&gt; valid.check(person.email, ...)\n\t}\nSynonyms:\nvalid.build3\nvalid build3","url":"valid.html#build3"},{"doc":"valid","title":"build4","content":"pub fn build4(\n  constructor: fn(a, b, c, d) -&gt; e,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; e, f)\n Build a validator for a type that has four attributes\nSynonyms:\nvalid.build4\nvalid build4","url":"valid.html#build4"},{"doc":"valid","title":"build5","content":"pub fn build5(\n  constructor: fn(a, b, c, d, e) -&gt; f,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; f, g)\n Build a validator for a type that has five attributes\nSynonyms:\nvalid.build5\nvalid build5","url":"valid.html#build5"},{"doc":"valid","title":"build6","content":"pub fn build6(\n  constructor: fn(a, b, c, d, e, f) -&gt; g,\n) -&gt; Result(\n  fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; g,\n  h,\n)\n Build a validator for a type that has six attributes\nSynonyms:\nvalid.build6\nvalid build6","url":"valid.html#build6"},{"doc":"valid","title":"check","content":"pub fn check(\n  accumulator: Result(fn(a) -&gt; b, #(c, List(c))),\n  value: d,\n  validator: fn(d) -&gt; Result(a, #(c, List(c))),\n) -&gt; Result(b, #(c, List(c)))\n Validate an attribute.\n\n ## Example\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build1(Person)\n\t\t|&gt; valid.check(person.name, valid.string_is_not_empty(ErrorEmpty))\n\t}\n\nSynonyms:\nvalid.check\nvalid check","url":"valid.html#check"},{"doc":"valid","title":"custom","content":"pub fn custom(\n  error: a,\n  assert_: fn(b) -&gt; Option(c),\n) -&gt; fn(b) -&gt; Result(c, #(a, List(a)))\n Create a custom validator\n\n A custom validator has two attributes:\n\n - The error\n - A check function\n\n The check function is a function that takes an `input` and returns `Option(output)`\n\n ## Example\n\n\tlet must_be_sam = fn(name: String) -&gt; Option(String) {\n\t\tcase name == &quot;Sam&quot; {\n\t\t\tTrue -&gt; Some(name)\n\t\t\tFalse -&gt; None\n\t\t}\n\t}\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build1(Person)\n\t\t|&gt; valid.check(person.name, valid.custom(&quot;Not Sam&quot;, must_be_sam))\n\t}\nSynonyms:\nvalid.custom\nvalid custom","url":"valid.html#custom"},{"doc":"valid","title":"int_max","content":"pub fn int_max(\n  error: a,\n  max: Int,\n) -&gt; fn(Int) -&gt; Result(Int, #(a, List(a)))\n\nSynonyms:\nvalid.int_max\nvalid int_max","url":"valid.html#int_max"},{"doc":"valid","title":"int_min","content":"pub fn int_min(\n  error: a,\n  min: Int,\n) -&gt; fn(Int) -&gt; Result(Int, #(a, List(a)))\n\nSynonyms:\nvalid.int_min\nvalid int_min","url":"valid.html#int_min"},{"doc":"valid","title":"is_some","content":"pub fn is_some(\n  error: a,\n) -&gt; fn(Option(b)) -&gt; Result(b, #(a, List(a)))\n Validate that a value is not None.\n Returns the value if Some.\n\n ## Example\n\n\ttype PersonInput { PersonInput(name: Option(String)) }\n\n\ttype PersonValid { PersonValid(name: String) }\n\n\tlet validator = fn(person) {\n\t\tvalid.build1(PersonValid)\n\t\t|&gt; valid.check(person.name, valid.is_some(&quot;Name is null&quot;))\n\t}\n\nSynonyms:\nvalid.is_some\nvalid is_some","url":"valid.html#is_some"},{"doc":"valid","title":"keep","content":"pub fn keep(\n  accumulator: Result(fn(a) -&gt; b, #(c, List(c))),\n  value: a,\n) -&gt; Result(b, #(c, List(c)))\n\n Keep a value as is.\n\n ## Example\n\n\tfn person_validor(person: Person) {\n\t\tvalid.build2(Person)\n\t\t\t|&gt; valid.check(person.name, ...)\n\t\t\t|&gt; valid.keep(person.age)\n\t}\n\nSynonyms:\nvalid.keep\nvalid keep","url":"valid.html#keep"},{"doc":"valid","title":"list_every","content":"pub fn list_every(\n  validator: fn(a) -&gt; Result(b, #(c, List(c))),\n) -&gt; fn(List(a)) -&gt; Result(List(b), #(c, List(c)))\n Validate a list of items.\n\n Run the given validator for each item returning all the errors.\n\n ## Example\n\n\ttype Collection = { Collection(items: List(String) ) }\n\n\tlet list_validator = valid.list_every(\n\t\tvalid.string_min_length(&quot;Must be at least 3&quot;, 3)\n\t)\n\n\tlet validator = fn(collection: Collection) {\n\t\tvalid.build1(Collection)\n\t\t|&gt; valid.check(collection.items, list_validator)\n\t}\nSynonyms:\nvalid.list_every\nvalid list_every","url":"valid.html#list_every"},{"doc":"valid","title":"list_is_not_empty","content":"pub fn list_is_not_empty(\n  error: a,\n) -&gt; fn(List(b)) -&gt; Result(List(b), #(a, List(a)))\n Validate that a list is not empty\nSynonyms:\nvalid.list_is_not_empty\nvalid list_is_not_empty","url":"valid.html#list_is_not_empty"},{"doc":"valid","title":"list_max_length","content":"pub fn list_max_length(\n  error: a,\n  max: Int,\n) -&gt; fn(List(b)) -&gt; Result(List(b), #(a, List(a)))\n Validate the max number of items in a list\nSynonyms:\nvalid.list_max_length\nvalid list_max_length","url":"valid.html#list_max_length"},{"doc":"valid","title":"list_min_length","content":"pub fn list_min_length(\n  error: a,\n  min: Int,\n) -&gt; fn(List(b)) -&gt; Result(List(b), #(a, List(a)))\n Validate the min number of items in a list\nSynonyms:\nvalid.list_min_length\nvalid list_min_length","url":"valid.html#list_min_length"},{"doc":"valid","title":"ok","content":"pub fn ok() -&gt; fn(a) -&gt; Result(a, #(b, List(b)))\n A validator that always succeeds\nSynonyms:\nvalid.ok\nvalid ok","url":"valid.html#ok"},{"doc":"valid","title":"optional","content":"pub fn optional(\n  validator: fn(a) -&gt; Result(b, #(c, List(c))),\n) -&gt; fn(Option(a)) -&gt; Result(Option(b), #(c, List(c)))\n Validate an optional value.\n\n Run the validator only if the value is Some.\n If the value is None then return None back.\n\n ## Example\n\n type PersonValid{\n   PersonValid(name: Option(String))\n }\n\n\tlet validator = fn(person) {\n\t\tvalid.build1(PersonValid)\n\t\t|&gt; valid.check(\n\t\t\tperson.name,\n\t\t\tvalid.optional(valid.string_min_length(&quot;Short&quot;, 3))\n\t\t)\n\t}\n\nSynonyms:\nvalid.optional\nvalid optional","url":"valid.html#optional"},{"doc":"valid","title":"optional_in","content":"pub fn optional_in(\n  get: fn(a) -&gt; Option(b),\n) -&gt; fn(a) -&gt; Result(Option(b), c)\n\nSynonyms:\nvalid.optional_in\nvalid optional_in","url":"valid.html#optional_in"},{"doc":"valid","title":"optional_in_dict","content":"pub fn optional_in_dict(\n  key: String,\n) -&gt; fn(Dict(String, a)) -&gt; Result(Option(a), b)\n\nSynonyms:\nvalid.optional_in_dict\nvalid optional_in_dict","url":"valid.html#optional_in_dict"},{"doc":"valid","title":"required_in","content":"pub fn required_in(\n  get: fn(a) -&gt; Option(b),\n  error: c,\n) -&gt; fn(a) -&gt; Result(b, #(c, List(c)))\n Validate an attribute required in an arbitrary data type\n Here you provide your own accessor\n The accessor should return `Option(property)`\n\n ## Example\n\n let get_name = fn(d) { dict.get(d, &quot;name&quot;) |&gt; option.from_result }\n\n\tlet validator = fn(dictionary: Dict(String, String)) {\n\t\tvalid.build1(Person)\n\t\t|&gt; valid.check_required(\n     from: dictionary,\n     get: get_name,\n     missing: &quot;Missing name&quot;,\n     validator: valid.string_is_not_empty(ErrorEmpty)\n   )\n\t}\n\nSynonyms:\nvalid.required_in\nvalid required_in","url":"valid.html#required_in"},{"doc":"valid","title":"required_in_dict","content":"pub fn required_in_dict(\n  key: String,\n  error: a,\n) -&gt; fn(Dict(String, b)) -&gt; Result(b, #(a, List(a)))\n Validate an attribute required in a dictionary\n If you have a dictionary instead of a custom type, use this.\n\n ## Example\n\n\tlet validator = fn(dictionary: Dict(String, String)) {\n\t\tvalid.build1(Person)\n\t\t|&gt; valid.check_required_in_dict(\n     from: dictionary,\n     get: &quot;name&quot;,\n     missing: &quot;Missing name&quot;,\n     validator: valid.string_is_not_empty(ErrorEmpty)\n   )\n\t}\n\nSynonyms:\nvalid.required_in_dict\nvalid required_in_dict","url":"valid.html#required_in_dict"},{"doc":"valid","title":"string_is_email","content":"pub fn string_is_email(\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate if a string is an email.\n\n This checks if a string follows a simple pattern `_@_`.\nSynonyms:\nvalid.string_is_email\nvalid string_is_email","url":"valid.html#string_is_email"},{"doc":"valid","title":"string_is_float","content":"pub fn string_is_float(\n  error: a,\n) -&gt; fn(String) -&gt; Result(Float, #(a, List(a)))\n Validate if a string parses to an Float. Returns the Float if so.\nSynonyms:\nvalid.string_is_float\nvalid string_is_float","url":"valid.html#string_is_float"},{"doc":"valid","title":"string_is_int","content":"pub fn string_is_int(\n  error: a,\n) -&gt; fn(String) -&gt; Result(Int, #(a, List(a)))\n Validate if a string parses to an Int. Returns the Int if so.\nSynonyms:\nvalid.string_is_int\nvalid string_is_int","url":"valid.html#string_is_int"},{"doc":"valid","title":"string_is_not_empty","content":"pub fn string_is_not_empty(\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate if a string is not empty\nSynonyms:\nvalid.string_is_not_empty\nvalid string_is_not_empty","url":"valid.html#string_is_not_empty"},{"doc":"valid","title":"string_max_length","content":"pub fn string_max_length(\n  error: a,\n  max: Int,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate the max length of a string\nSynonyms:\nvalid.string_max_length\nvalid string_max_length","url":"valid.html#string_max_length"},{"doc":"valid","title":"string_min_length","content":"pub fn string_min_length(\n  error: a,\n  min: Int,\n) -&gt; fn(String) -&gt; Result(String, #(a, List(a)))\n Validate the min length of a string\nSynonyms:\nvalid.string_min_length\nvalid string_min_length","url":"valid.html#string_min_length"},{"doc":"valid","title":"whole","content":"pub fn whole(\n  validator: fn(a) -&gt; Result(a, b),\n) -&gt; fn(Result(a, #(b, List(b)))) -&gt; Result(a, #(b, List(b)))\n Validate a structure as a whole.\n\n Sometimes we need to validate a property in relation to another.\n\n This function requires a check function like:\n\n\tfn(a) -&gt; Result(a, error)\n\n ## Example\n\n\tlet strengh_and_level_validator = fn(c: Character) {\n\t\tcase c.level &gt; c.strength {\n\t\t\tTrue -&gt; Error(error)\n\t\t\tFalse -&gt; Ok(c)\n\t\t}\n\t}\n\n\tlet validator = fn(c: Character) {\n\t\tvalid.build2(Character)\n\t\t|&gt; valid.check(c.level, valid.int_min(&quot;Level must be more that zero&quot;, 1))\n\t\t|&gt; valid.check(c.strength, valid.int_min(&quot;Strength must be more that zero&quot;, 1))\n\t\t|&gt; valid.whole(strengh_and_level_validator)\n\t}\n\nSynonyms:\nvalid.whole\nvalid whole","url":"valid.html#whole"},{"doc":"valid","title":"valid","content":"","url":"valid.html"}]);